class Constant:
    DataType = "datatype"
    DataTypes = "datatypes"
    Expressions = "expressions"
    Identifiers = "identifiers"
    Identifier = "identifier"
    Package = "package"
    Class = "class"
    Name = "name"
    DefaultIdentifier = "identifier_jdefault_0063"
    Parent = "parent"

